from oddrn_generator.path_models import (
    BasePathsModel,
    PostgresqlPathsModel,
    MysqlPathsModel,
    KafkaPathsModel,
    GluePathsModel,
    SagemakerPathsModel,
    SnowflakePathsModel,
    AirflowPathsModel,
    HivePathsModel,
    DynamodbPathsModel,
    OdbcPathsModel,
    MssqlPathsModel,
    OraclePathsModel,
    RedshiftPathsModel,
    ClickHousePathsModel,
    KafkaConnectorPathsModel,
    AthenaPathsModel,
    QuicksightPathsModel,
    DbtPathsModel,
    TableauPathsModel,
    PrefectPathsModel,
    Neo4jPathsModel,
    S3PathsModel,
    ElasticSearchPathsModel,
    CassandraPathsModel,
    FeastPathsModel,
    KubeflowPathsModel,
    TarantoolPathsModel,
    KinesisPathsModel,
    MongoPathsModel,
    VerticaPathsModel
)
from oddrn_generator.server_models import (
    AbstractServerModel,
    AWSCloudModel,
    HostnameModel,
)


class Generator:
    source: str = None
    server_model = None
    paths_model = None

    def __new__(cls, *args, **kwargs):
        if not kwargs.get("data_source"):
            return super(Generator, cls).__new__(cls)
        subclass = {subclass.source: subclass for subclass in cls.__subclasses__()}.get(
            kwargs["data_source"]
        )
        if not subclass:
            raise ValueError("data_source is invalid")
        return super(Generator, subclass).__new__(subclass)

    def __init__(
        self,
        *,
        data_source=None,
        cloud_settings: dict = None,
        host_settings: str = None,
        **path_attributes,
    ):
        if cloud_settings:
            server_settings = cloud_settings
        elif host_settings:
            server_settings = {"host": host_settings}
        else:
            raise ValueError(
                "You must specify at least one parameter: 'cloud_settings' or 'host_settings'"
            )
        self.server_obj: AbstractServerModel = self.__build_server(server_settings)
        self.paths_obj: BasePathsModel = self.__build_paths(**path_attributes)

    def __build_server(self, server: dict) -> AbstractServerModel:
        return self.server_model(**server)

    def __build_paths(self, **paths) -> BasePathsModel:
        path_obj: BasePathsModel = self.paths_model(**paths)
        path_obj.validate_all_paths()
        return path_obj

    @property
    def base_oddrn(self) -> str:
        return f"//{self.source}/{self.server_obj}"

    @property
    def available_paths(self) -> tuple:
        return tuple(self.paths_obj.__config__.dependencies_map.keys())

    def get_oddrn_by_path(self, path: str, new_value: str = None) -> str:
        dependency = self.paths_obj.get_dependency(path)
        if new_value:
            self.paths_obj.set_path_value(path, new_value)
        else:
            self.paths_obj.check_if_path_is_set(path)
        paths_dict = self.paths_obj.dict(
            include=set(dependency), exclude_none=True, by_alias=True
        )
        return (
            f"{self.base_oddrn}/{'/'.join([f'{k}/{v}' for k, v in paths_dict.items()])}"
        )

    def set_oddrn_paths(self, **new_paths) -> None:
        old_paths = {
            k: v
            for k, v in self.paths_obj.dict(exclude_none=True).items()
            if k not in list(new_paths.keys())
        }

        self.paths_obj = self.__build_paths(**old_paths, **new_paths)

    def get_data_source_oddrn(self):
        return (
            self.get_oddrn_by_path(self.paths_obj.data_source_path)
            if self.paths_obj.data_source_path
            else self.base_oddrn
        )


class PostgresqlGenerator(Generator):
    source = "postgresql"
    paths_model = PostgresqlPathsModel
    server_model = HostnameModel


class GlueGenerator(Generator):
    source = "glue"
    paths_model = GluePathsModel
    server_model = AWSCloudModel


class MysqlGenerator(Generator):
    source = "mysql"
    paths_model = MysqlPathsModel
    server_model = HostnameModel


class KafkaGenerator(Generator):
    source = "kafka"
    paths_model = KafkaPathsModel
    server_model = HostnameModel


class KafkaConnectGenerator(Generator):
    source = "kafkaconnect"
    paths_model = KafkaConnectorPathsModel
    server_model = HostnameModel


class SnowflakeGenerator(Generator):
    source = "snowflake"
    paths_model = SnowflakePathsModel
    server_model = HostnameModel


class AirflowGenerator(Generator):
    source = "airflow"
    paths_model = AirflowPathsModel
    server_model = HostnameModel


class HiveGenerator(Generator):
    source = "hive"
    paths_model = HivePathsModel
    server_model = HostnameModel


class ElasticSearchGenerator(Generator):
    source = "elasticsearch"
    paths_model = ElasticSearchPathsModel
    server_model = HostnameModel


class FeastGenerator(Generator):
    source = "feast"
    paths_model = FeastPathsModel
    server_model = HostnameModel


class DynamodbGenerator(Generator):
    source = "dynamodb"
    paths_model = DynamodbPathsModel
    server_model = AWSCloudModel


class OdbcGenerator(Generator):
    source = "odbc"
    paths_model = OdbcPathsModel
    server_model = HostnameModel


class MssqlGenerator(Generator):
    source = "mssql"
    paths_model = MssqlPathsModel
    server_model = HostnameModel


class OracleGenerator(Generator):
    source = "oracle"
    paths_model = OraclePathsModel
    server_model = HostnameModel


class RedshiftGenerator(Generator):
    source = "redshift"
    paths_model = RedshiftPathsModel
    server_model = HostnameModel


class ClickHouseGenerator(Generator):
    source = "clickhouse"
    paths_model = ClickHousePathsModel
    server_model = HostnameModel


class AthenaGenerator(Generator):
    source = "athena"
    paths_model = AthenaPathsModel
    server_model = AWSCloudModel


class QuicksightGenerator(Generator):
    source = "quicksight"
    paths_model = QuicksightPathsModel
    server_model = AWSCloudModel


class DbtGenerator(Generator):
    source = "dbt"
    paths_model = DbtPathsModel
    server_model = HostnameModel


class TableauGenerator(Generator):
    source = "tableau"
    paths_model = TableauPathsModel
    server_model = HostnameModel


class PrefectGenerator(Generator):
    source = "prefect"
    paths_model = PrefectPathsModel
    server_model = HostnameModel


class Neo4jGenerator(Generator):
    source = "neo4j"
    paths_model = Neo4jPathsModel
    server_model = HostnameModel


class S3Generator(Generator):
    source = "s3"
    paths_model = S3PathsModel
    server_model = AWSCloudModel


class CassandraGenerator(Generator):
    source = "cassandra"
    paths_model = CassandraPathsModel
    server_model = HostnameModel


class SagemakerGenerator(Generator):
    source = "sagemaker"
    paths_model = SagemakerPathsModel
    server_model = AWSCloudModel


class KinesisGenerator(Generator):
    source = "kinesis"
    paths_model = KinesisPathsModel
    server_model = AWSCloudModel


class KubeflowGenerator(Generator):
    source = "kubeflow"
    paths_model = KubeflowPathsModel
    server_model = HostnameModel


class TarantoolGenerator(Generator):
    source = "tarantool"
    paths_model = TarantoolPathsModel
    server_model = HostnameModel


class MongoGenerator(Generator):
    source = "mongo"
    paths_model = MongoPathsModel
    server_model = HostnameModel


class VerticaGenerator(Generator):
    source = "vertica"
    paths_model = VerticaPathsModel
    server_model = HostnameModel


#
#
# class DVCGenerator(Generator):  # todo:
#     source = "dvc"
#     paths_model = DVCPathsModel
#     server_model = HostnameModel
#
#
# class GreatExpectationsGenerator(Generator):  # todo:
#     source = "great_expectations"
#     paths_model = GreatExpectationsPathsModel
#     server_model = AWSCloudModel



from typing import Optional

from pydantic import BaseModel, Field

from oddrn_generator.exceptions import (
    WrongPathOrderException,
    PathDoestExistException,
    EmptyPathValueException,
)


class BasePathsModel(BaseModel):
    class Config:
        dependencies_map = {}
        data_source_path = None
        allows_null = []
        extra = "forbid"
        allow_population_by_field_name = True

    def __validate_path(self, field) -> None:
        for deps in reversed(self.__config__.dependencies_map[field]):
            deps_value = getattr(self, deps, None)
            # allow dependency null if it is in allow_null list
            if deps_value is None and deps in self.__config__.allows_null:
                return
            if not deps_value:
                raise WrongPathOrderException(
                    f"'{field}' can not be without '{deps}' attribute"
                )

    def validate_all_paths(self) -> None:
        for field in self.__fields_set__:
            self.__validate_path(field)

    def get_dependency(self, field) -> tuple:
        dependency = self.__config__.dependencies_map.get(field)
        if not dependency:
            raise PathDoestExistException(f"Path '{field}' doesn't exist in generator")
        return dependency

    def check_if_path_is_set(self, path: str) -> None:
        if not getattr(self, path, None):
            raise EmptyPathValueException(f"Path '{path}' is not set up")

    def set_path_value(self, path: str, value: str) -> None:
        setattr(self, path, value)
        self.__validate_path(path)

    @property
    def data_source_path(self):
        return self.__config__.data_source_path


class PostgresqlPathsModel(BasePathsModel):
    databases: str
    schemas: Optional[str]
    databases: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "schemas": (
                "databases",
                "schemas",
            ),
            "tables": ("databases", "schemas", "tables"),
            "views": ("databases", "schemas", "views"),
            "tables_columns": ("databases", "schemas", "tables", "tables_columns"),
            "views_columns": ("databases", "schemas", "views", "views_columns"),
        }
        data_source_path = "databases"


class MysqlPathsModel(BasePathsModel):
    databases: str
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "tables": ("databases", "tables"),
            "views": ("databases", "views"),
            "tables_columns": ("databases", "tables", "tables_columns"),
            "views_columns": ("databases", "views", "views_columns"),
        }
        data_source_path = "databases"


class KafkaPathsModel(BasePathsModel):
    topics: Optional[str]

    class Config:
        dependencies_map = {
            "topics": ("topics",),
        }


class KafkaConnectorPathsModel(BasePathsModel):
    connectors: str

    class Config:
        dependencies_map = {
            "connectors": ("connectors",),
        }


class GluePathsModel(BasePathsModel):
    databases: Optional[str]
    tables: Optional[str]
    columns: Optional[str]
    owners: Optional[str]
    jobs: Optional[str]
    runs: Optional[str]

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "tables": ("databases", "tables"),
            "columns": ("databases", "tables", "columns"),
            "owners": ("owners",),
            "jobs": ("jobs",),
            "runs": ("jobs", "runs"),
        }


class SnowflakePathsModel(BasePathsModel):
    warehouses: str
    databases: Optional[str]
    schemas: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")
    owners: Optional[str]

    class Config:
        dependencies_map = {
            "warehouses": ("warehouses",),
            "databases": ("warehouses", "databases"),
            "schemas": ("warehouses", "databases", "schemas"),
            "tables": ("warehouses", "databases", "schemas", "tables"),
            "views": ("warehouses", "databases", "schemas", "views"),
            "tables_columns": (
                "warehouses",
                "databases",
                "schemas",
                "tables",
                "tables_columns",
            ),
            "views_columns": (
                "warehouses",
                "databases",
                "schemas",
                "views",
                "views_columns",
            ),
            "owners": ("warehouses", "databases", "owners"),
        }
        data_source_path = "warehouses"


class AirflowPathsModel(BasePathsModel):
    dags: Optional[str]
    tasks: Optional[str]
    runs: Optional[str]

    class Config:
        dependencies_map = {
            "dags": ("dags",),
            "tasks": ("dags", "tasks"),
            "runs": ("dags", "tasks", "runs"),
        }


class HivePathsModel(BasePathsModel):
    databases: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")
    owners: Optional[str]

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "tables": ("databases", "tables"),
            "views": ("databases", "views"),
            "tables_columns": ("databases", "tables", "tables_columns"),
            "views_columns": ("databases", "views", "views_columns"),
            "owners": ("owners",),
        }
        data_source_path = "databases"


class ElasticSearchPathsModel(BasePathsModel):
    indexes: Optional[str]
    fields: Optional[str]

    class Config:
        dependencies_map = {
            "indexes": ("indexes",),
            "fields": ("indexes", "fields"),
        }


class FeastPathsModel(BasePathsModel):
    featureviews: Optional[str]
    features: Optional[str]
    subfeatures: Optional[str]

    class Config:
        dependencies_map = {
            "featureviews": ("featureviews",),
            "features": ("featureviews", "features"),
            "subfeatures": ("featureviews", "features", "subfeatures"),
        }


class DynamodbPathsModel(BasePathsModel):
    tables: Optional[str]
    columns: Optional[str]

    class Config:
        dependencies_map = {
            "tables": ("tables",),
            "columns": ("tables", "columns"),
        }


class OdbcPathsModel(BasePathsModel):
    databases: str
    schemas: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "schemas": ("databases", "schemas"),
            "tables": ("databases", "schemas", "tables"),
            "views": ("databases", "schemas", "views"),
            "tables_columns": ("databases", "schemas", "tables", "tables_columns"),
            "views_columns": ("databases", "schemas", "views", "views_columns"),
        }
        data_source_path = "databases"


class MssqlPathsModel(BasePathsModel):
    databases: str
    schemas: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "schemas": ("databases", "schemas"),
            "tables": ("databases", "schemas", "tables"),
            "views": ("databases", "schemas", "views"),
            "tables_columns": ("databases", "schemas", "tables", "tables_columns"),
            "views_columns": ("databases", "schemas", "views", "views_columns"),
        }
        data_source_path = "databases"


class OraclePathsModel(BasePathsModel):
    schemas: str
    databases: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    columns: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "schemas": ("schemas",),
            "databases": ("schemas", "databases"),
            "tables": ("schemas", "databases", "tables"),
            "views": ("schemas", "databases", "views"),
            "tables_columns": ("schemas", "databases", "tables", "tables_columns"),
            "views_columns": ("schemas", "databases", "views", "views_columns"),
        }
        data_source_path = "databases"


class RedshiftPathsModel(BasePathsModel):
    databases: str
    schemas: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "schemas": ("databases", "schemas"),
            "tables": ("databases", "schemas", "tables"),
            "views": ("databases", "schemas", "views"),
            "tables_columns": ("databases", "schemas", "tables", "tables_columns"),
            "views_columns": ("databases", "schemas", "views", "views_columns"),
        }
        data_source_path = "databases"


class ClickHousePathsModel(BasePathsModel):
    databases: str
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "tables": ("databases", "tables"),
            "views": ("databases", "views"),
            "tables_columns": ("databases", "tables", "tables_columns"),
            "views_columns": ("databases", "views", "views_columns"),
        }
        data_source_path = "databases"


class AthenaPathsModel(BasePathsModel):
    catalogs: Optional[str]
    databases: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "catalogs": ("catalogs",),
            "databases": ("catalogs", "databases"),
            "tables": ("catalogs", "databases", "tables"),
            "views": ("catalogs", "databases", "views"),
            "tables_columns": ("catalogs", "databases", "tables", "tables_columns"),
            "views_columns": ("catalogs", "databases", "views", "views_columns"),
        }


class QuicksightPathsModel(BasePathsModel):
    datasets: Optional[str]
    analyses: Optional[str]
    dashboards: Optional[str]
    data_sources: Optional[str]

    class Config:
        dependencies_map = {
            "datasets": ("datasets",),
            "analyses": ("analyses",),
            "dashboards": ("dashboards",),
            "data_sources": ("data_sources",),
        }


class DbtPathsModel(BasePathsModel):
    databases: Optional[str]
    schemas: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "schemas": ("databases", "schemas"),
            "tables": ("databases", "schemas", "tables"),
            "views": ("databases", "schemas", "views"),
            "tables_columns": ("databases", "schemas", "tables", "tables_columns"),
            "views_columns": ("databases", "schemas", "views", "views_columns"),
        }


class PrefectPathsModel(BasePathsModel):
    flows: str
    tasks: Optional[str]
    runs: Optional[str]

    class Config:
        dependencies_map = {
            "flows": ("flows",),
            "tasks": ("flows", "tasks"),
            "runs": ("flows", "tasks", "runs"),
        }


class TableauPathsModel(BasePathsModel):
    sites: str
    databases: Optional[str]
    schemas: Optional[str]
    tables: Optional[str]
    columns: Optional[str]
    workbooks: Optional[str]
    sheets: Optional[str]

    class Config:
        dependencies_map = {
            "sites": ("sites",),
            "databases": (
                "sites",
                "databases",
            ),
            "schemas": ("sites", "databases", "schemas"),
            "tables": ("sites", "databases", "schemas", "tables"),
            "columns": ("sites", "databases", "schemas", "tables", "columns"),
            "workbooks": ("sites", "workbooks"),
            "sheets": ("sites", "workbooks", "sheets"),
        }
        data_source_path = "sites"
        allows_null = ["schemas"]


class Neo4jPathsModel(BasePathsModel):
    databases: str
    nodes: Optional[str]
    fields: Optional[str]

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "nodes": ("databases", "nodes"),
            "fields": ("databases", "nodes", "fields"),
        }
        data_source_path = "databases"


class S3PathsModel(BasePathsModel):
    buckets: Optional[str]
    keys: Optional[str]
    columns: Optional[str]

    class Config:
        dependencies_map = {
            "buckets": ("buckets",),
            "keys": ("buckets", "keys"),
            "columns": ("buckets", "keys", "columns"),
        }


class CassandraPathsModel(BasePathsModel):
    keyspaces: str
    tables: Optional[str]
    columns: Optional[str]

    class Config:
        dependencies_map = {
            "keyspaces": ("keyspaces",),
            "tables": ("keyspaces", "tables"),
            "columns": ("keyspaces", "tables", "columns"),
        }
        data_source_path = "keyspaces"


class SagemakerPathsModel(BasePathsModel):
    experiments: Optional[str]
    trials: Optional[str]
    jobs: Optional[str]
    artifacts: Optional[str]

    class Config:
        dependencies_map = {
            "experiments": ("experiments",),
            "trials": ("experiments", "trials"),
            "jobs": ("experiments", "trials", "jobs"),
            "artifacts": ("experiments", "trials", "artifacts"),
        }


class KubeflowPathsModel(BasePathsModel):
    pipelines: Optional[str]
    experiments: Optional[str]
    runs: Optional[str]

    class Config:
        dependencies_map = {
            "pipelines": ("pipelines",),
            "experiments": ("experiments",),
            "runs": (
                "experiments",
                "runs",
            ),
        }


class TarantoolPathsModel(BasePathsModel):
    spaces: Optional[str]
    columns: Optional[str]

    class Config:
        dependencies_map = {
            "spaces": ("spaces",),
            "columns": ("spaces", "columns"),
        }


class KinesisPathsModel(BasePathsModel):
    streams: Optional[str]
    shards: Optional[str]
    data_records: Optional[str]

    class Config:
        dependencies_map = {
            'streams': ('streams',),
            'shards': ('streams', 'shards'),
            'data_records': ('streams', 'shards', 'data_records')
        }


class MongoPathsModel(BasePathsModel):
    databases: str
    collections: Optional[str]
    columns: Optional[str]

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "collections": ("databases", "collections"),
            "columns": ("databases", "collections", "columns"),
        }
        data_source_path = "databases"


class VerticaPathsModel(BasePathsModel):
    databases: str
    schemas: Optional[str]
    tables: Optional[str]
    views: Optional[str]
    tables_columns: Optional[str] = Field(alias="columns")
    views_columns: Optional[str] = Field(alias="columns")

    class Config:
        dependencies_map = {
            "databases": ("databases",),
            "schemas": ("databases", "schemas"),
            "tables": ("databases", "schemas", "tables"),
            "views": ("databases", "schemas", "views"),
            "tables_columns": ("databases", "schemas", "tables", "tables_columns"),
            "views_columns": ("databases", "schemas", "views", "views_columns"),
        }
        data_source_path = "databases"

# class DVCPathsModel(BasePathsModel):  # todo:
#     pass
#
#
# class GreatExpectationsPathsModel(BasePathsModel):  # todo:
#     suits: Optional[str]
#     runs: Optional[str]
#     suits_types: Optional[str] = Field(alias='types')
#     runs_types: Optional[str] = Field(alias='types')
#
#     class Config:
#         dependencies_map = {
#             'suits':        ('suits',),
#             'suits_types':  ('suits', 'suits_types'),
#             'runs':         ('runs',),
#             'runs_types':   ('runs', 'runs_types'),
#         }



from abc import ABC, abstractmethod

from pydantic import BaseModel


class AbstractServerModel(ABC):
    @abstractmethod
    def __str__(self) -> str:
        raise NotImplementedError


class HostnameModel(AbstractServerModel, BaseModel):
    host: str

    def __str__(self) -> str:
        return f"host/{self.host}"


class AWSCloudModel(AbstractServerModel, BaseModel):
    account: str
    region: str

    def __str__(self) -> str:
        return f"cloud/aws/{'/'.join('{}/{}'.format(*p) for p in self.dict().items())}"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['oddrn_generator']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.8.2,<2.0.0']

setup_kwargs = {
    'name': 'oddrn-generator',
    'version': '0.1.39',
    'description': 'Open Data Discovery Resource Name Generator',
    'long_description': '[![PyPI version](https://badge.fury.io/py/oddrn-generator.svg)](https://badge.fury.io/py/oddrn-generator)\n# Open Data Discovery Resource Name Generator\n## Requirements\n* __Python >= 3.7__\n## Installation\n```bash\npoetry add oddrn-generator\n# or\npip install oddrn-generator\n```\n\n## Usage and configuration\n### Available generators\n* cassandra - CassandraGenerator\n* postgresql - PostgresqlGenerator\n* mysql - MysqlGenerator\n* glue - GlueGenerator\n* s3 - S3Generator\n* kafka - KafkaGenerator\n* kafkaconnect - KafkaConnectGenerator\n* snowflake - SnowflakeGenerator\n* airflow - AirflowGenerator\n* hive - HiveGenerator\n* dynamodb - DynamodbGenerator\n* odbc - OdbcGenerator\n* mssql - MssqlGenerator\n* oracle - OracleGenerator\n* redshift - RedshiftGenerator\n* clickhouse - ClickHouseGenerator\n* athena - AthenaGenerator\n* quicksight - QuicksightGenerator\n* dbt - DbtGenerator\n* prefect - PrefectGenerator\n* tableau - TableauGenerator\n* neo4j - Neo4jGenerator\n* mongodb - MongoGenerator\n* vertica - VerticaGenerator\n### Work in progress generators\n* kubeflow - KubeflowGenerator\n* dvc - DVCGenerator\n* great_expectations - GreatExpectationsGenerator\n\n### Generator properties\n* base_oddrn - Get base oddrn (without path)\n* available_paths - Get all available path of generator \n\n### Generator methods\n* get_oddrn_by_path(path_name, new_value=None) - Get oddrn string by path. You also can set value for this path using \'new_value\' param\n* set_oddrn_paths(**kwargs) - Set or update values of oddrn path\n* get_data_source_oddrn() - Get data source oddrn \n\n### Generator parameters:\n* host_settings: str - optional. Hostname configuration\n* cloud_settings: dict - optional.  Cloud configuration\n* **kwargs - path\'s name and values\n\n### Example usage\n```python\n# postgresql\nfrom oddrn_generator import PostgresqlGenerator\noddrn_gen = PostgresqlGenerator(\n  host_settings=\'my.host.com:5432\', \n  schemas=\'schema_name\', databases=\'database_name\', tables=\'table_name\'\n)\n\noddrn_gen.base_oddrn\n# //postgresql/host/my.host.com:5432\noddrn_gen.available_paths\n# (\'schemas\', \'databases\', \'tables\', \'columns\')\n\noddrn_gen.get_data_source_oddrn()\n# //postgresql/host/my.host.com:5432/schemas/schema_name/databases/database_name\n\noddrn_gen.get_oddrn_by_path("schemas")\n# //postgresql/host/my.host.com:5432/schemas/schema_name\n\noddrn_gen.get_oddrn_by_path("databases")\n# //postgresql/host/my.host.com:5432/schemas/schema_name/databases/database_name\n\noddrn_gen.get_oddrn_by_path("tables")\n# //postgresql/host/my.host.com:5432/schemas/schema_name/databases/database_name/tables/table_name\n\n# you can set or change path:\noddrn_gen.set_oddrn_paths(tables=\'another_table_name\', columns=\'new_column_name\')\noddrn_gen.get_oddrn_by_path("columns")\n# //postgresql/host/my.host.com:5432/schemas/schema_name/databases/database_name/tables/another_table_name/columns/new_column_name\n\n# you can get path wih new values:\noddrn_gen.get_oddrn_by_path("columns", new_value="another_new_column_name")\n# //postgresql/host/my.host.com:5432/schemas/schema_name/databases/database_name/tables/another_table_name/columns/another_new_column_name\n\n\n# glue\nfrom oddrn_generator import GlueGenerator\noddrn_gen = GlueGenerator(\n  cloud_settings={\'account\': \'acc_id\', \'region\':\'reg_id\'}, \n  databases=\'database_name\', tables=\'table_name\', columns=\'column_name\', \n  jobs=\'job_name\', runs=\'run_name\', owners=\'owner_name\'\n)\n\noddrn_gen.available_paths\n# (\'databases\', \'tables\', \'columns\', \'owners\', \'jobs\', \'runs\')\n\noddrn_gen.get_oddrn_by_path("databases")\n# //glue/cloud/aws/account/acc_id/region/reg_id/databases/database_name\n\noddrn_gen.get_oddrn_by_path("tables")\n# //glue/cloud/aws/account/acc_id/region/reg_id/databases/database_name/tables/table_name\'\n\noddrn_gen.get_oddrn_by_path("columns")\n# //glue/cloud/aws/account/acc_id/region/reg_id/databases/database_name/tables/table_name/columns/column_name\n\noddrn_gen.get_oddrn_by_path("jobs")\n# //glue/cloud/aws/account/acc_id/region/reg_id/jobs/job_name\n\noddrn_gen.get_oddrn_by_path("runs")\n# //glue/cloud/aws/account/acc_id/region/reg_id/jobs/job_name/runs/run_name\n\noddrn_gen.get_oddrn_by_path("owners")\n# //glue/cloud/aws/account/acc_id/region/reg_id/owners/owner_name\n\n```\n\n### Exceptions\n* WrongPathOrderException - raises when trying set path that depends on another path\n```python\nfrom oddrn_generator import PostgresqlGenerator\noddrn_gen = PostgresqlGenerator(\n    host_settings=\'my.host.com:5432\', \n    schemas=\'schema_name\', databases=\'database_name\',\n    columns=\'column_without_table\'\n)\n# WrongPathOrderException: \'columns\' can not be without \'tables\' attribute\n```\n* EmptyPathValueException - raises when trying to get a path that is not set up\n```python\nfrom oddrn_generator import PostgresqlGenerator\noddrn_gen = PostgresqlGenerator(\n    host_settings=\'my.host.com:5432\', schemas=\'schema_name\', databases=\'database_name\',\n)\noddrn_gen.get_oddrn_by_path("tables")\n\n# EmptyPathValueException: Path \'tables\' is not set up\n```\n* PathDoestExistException - raises when trying to get not existing oddrn path\n```python\nfrom oddrn_generator import PostgresqlGenerator\noddrn_gen = PostgresqlGenerator(\n    host_settings=\'my.host.com:5432\', schemas=\'schema_name\', databases=\'database_name\',\n)\noddrn_gen.get_oddrn_by_path("jobs")\n\n# PathDoestExistException: Path \'jobs\' doesn\'t exist in generator\n```\n\n## Development\n```bash\n#Install dependencies\npoetry install\n\n#Activate shell\npoetry shell\n\n# Run tests\npython run pytest\n```\n',
    'author': 'Open Data Discovery',
    'author_email': 'pypi@opendatadiscovery.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/opendatadiscovery/oddrn-generator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from oddrn_generator.generators import (
    Generator,
    PostgresqlGenerator,
    MysqlGenerator,
    KafkaGenerator,
    KafkaConnectGenerator,
    GlueGenerator,
    SnowflakeGenerator,
    AirflowGenerator,
    HiveGenerator,
    DynamodbGenerator,
    OdbcGenerator,
    MssqlGenerator,
    OracleGenerator,
    RedshiftGenerator,
    ClickHouseGenerator,
    AthenaGenerator,
    QuicksightGenerator,
    DbtGenerator,
    TableauGenerator,
    PrefectGenerator,
    Neo4jGenerator,
    ElasticSearchGenerator,
    SagemakerGenerator,
    FeastGenerator,
    KubeflowGenerator,
    TarantoolGenerator,
    KinesisGenerator,
    MongoGenerator,
    VerticaGenerator,
)

__all__ = [
    "AirflowGenerator",
    "AthenaGenerator",
    "ClickHouseGenerator",
    "DbtGenerator",
    "DynamodbGenerator",
    "ElasticSearchGenerator",
    "FeastGenerator",
    "Generator",
    "GlueGenerator",
    "HiveGenerator",
    "KafkaConnectGenerator",
    "KafkaGenerator",
    "KubeflowGenerator",
    "MssqlGenerator",
    "MysqlGenerator",
    "Neo4jGenerator",
    "OdbcGenerator",
    "OracleGenerator",
    "PostgresqlGenerator",
    "PrefectGenerator",
    "QuicksightGenerator",
    "RedshiftGenerator",
    "SagemakerGenerator",
    "SnowflakeGenerator",
    "TableauGenerator",
    "TarantoolGenerator",
    "KinesisGenerator",
    "MongoGenerator",
    "VerticaGenerator",
]
